package sun.awt.datatransfer;

import java.lang.reflect.Proxy;
import java.util.HashSet;
import java.io.ObjectStreamClass;
import java.io.IOException;
import java.io.InputStream;
import java.util.Set;
import java.util.Map;
import java.io.ObjectInputStream;

final class ClassLoaderObjectInputStream extends ObjectInputStream
{
    private final Map<Set<String>, ClassLoader> map;
    
    ClassLoaderObjectInputStream(InputStream in, Map<Set<String>, ClassLoader> map) throws IOException {
        super(in);
        if (map == null) {
            throw new NullPointerException("Null map");
        }
        this.map = map;
        return;
    }
    
    @Override
    protected Class<?> resolveClass(ObjectStreamClass desc) throws IOException, ClassNotFoundException {
        String name = desc.getName();
        HashSet set = new HashSet(1);
        set.add(name);
        ClassLoader loader = (ClassLoader)this.map.get(set);
        if (loader != null) {
            return Class.forName(name, false, loader);
        }
        return super.resolveClass(desc);
    }
    
    @Override
    protected Class<?> resolveProxyClass(String[] interfaces) throws IOException, ClassNotFoundException {
        HashSet set = new HashSet(interfaces.length);
        for (int i = 0; i < interfaces.length; ++i) {
            set.add(interfaces[i]);
        }
        ClassLoader loader = (ClassLoader)this.map.get(set);
        if (loader == null) {
            return super.resolveProxyClass(interfaces);
        }
        ClassLoader classLoader = null;
        int n = 0;
        Class[] interfaces2 = new Class[interfaces.length];
        Class<Object> forName;
        for (int j = 0; j < interfaces.length; interfaces2[j] = forName, ++j) {
            forName = Class.forName(interfaces[j], false, loader);
            if ((forName.getModifiers() & 0x1) == 0x0) {
                if (n != 0) {
                    if (classLoader != forName.getClassLoader()) {
                        throw new IllegalAccessError("conflicting non-public interface class loaders");
                    }
                }
                else {
                    classLoader = forName.getClassLoader();
                    n = 1;
                }
            }
        }
        try {
            return Proxy.getProxyClass((n != 0) ? classLoader : loader, interfaces2);
        }
        catch (IllegalArgumentException ex) {
            throw new ClassNotFoundException(null, ex);
        }
    }
}
